@@ -205,7 +205,8 @@ def _export_columnize_data(obj: typing.Any, default_column_name: str) -> \
 
         if len({str(x) for x in obj.keys()}) != len(obj.columns):
             raise SBDFError("obj does not have unique column names")
-        columns = obj.to_dict("list")
+        # columns = obj.to_dict("list")
+        columns = obj
         column_names = obj.columns.tolist()
         column_types = {str(k): _ValueTypeId.infer_from_dtype(v, f"column '{str(k)}'") for (k, v) in obj.iteritems()}
     elif isinstance(obj, pd.Series):
@@ -303,21 +304,36 @@ def _export_column_metadata(columns: typing.Dict[str, typing.List], column_names
 def _export_table_slices(columns: typing.Dict[str, typing.List], column_names: typing.List[str],
                          column_types: typing.Dict[str, '_ValueTypeId'], file: typing.BinaryIO,
                          row_count: int, tmeta: '_TableMetadata') -> None:
-    max_rows_per_slice = max(10, 100000 // max(1, len(column_names)))
+    # max_rows_per_slice = max(10, 100000 // max(1, len(column_names)))
+    max_rows_per_slice = 50000
     row_offset = 0
     while row_offset < row_count:
         slice_row_count = min(max_rows_per_slice, row_count - row_offset)
         tslice = _TableSlice(tmeta)
         for colname in column_names:
-            obj = _SbdfObject(column_types.get(str(colname)), columns[colname][row_offset:row_offset + slice_row_count])
+            if isinstance(columns, pd.DataFrame):
+                dataslice = columns.loc[row_offset:row_offset + slice_row_count, colname]
+            else:
+                dataslice = columns[colname][row_offset:row_offset + slice_row_count]
+            obj = _SbdfObject(column_types.get(str(colname)), dataslice)
             cslice = _ColumnSlice(_ValueArray(_ValueArrayEncoding.PLAIN_ARRAY, obj))
-            invalid = [pd.isnull(x) for x in obj.data]
-            if any(invalid):
-                obj_vt = _ValueType(column_types.get(str(colname)))
-                obj.data = [obj_vt.missing_value() if missing else val for val, missing in zip(obj.data, invalid)]
-                obj_empty = _SbdfObject(_ValueTypeId.BOOL, invalid)
-                va_empty = _ValueArray(_ValueArrayEncoding.BIT_ARRAY, obj_empty)
-                cslice.add_property(_ColumnSlice.ValueProperty_IsInvalid, va_empty)
+
+            if isinstance(dataslice, pd.Series):
+                invalid = dataslice.isna()
+                if invalid.sum() > 0:
+                    obj_vt = _ValueType(column_types.get(str(colname)))
+                    dataslice = dataslice.fillna(obj_vt.missing_value())
+                    obj_empty = _SbdfObject(_ValueTypeId.BOOL, invalid.tolist())
+                    va_empty = _ValueArray(_ValueArrayEncoding.BIT_ARRAY, obj_empty)
+                    cslice.add_property(_ColumnSlice.ValueProperty_IsInvalid, va_empty)
+            else:
+                invalid = [pd.isnull(x) for x in obj.data]
+                if any(invalid):
+                    obj_vt = _ValueType(column_types.get(str(colname)))
+                    obj.data = [obj_vt.missing_value() if missing else val for val, missing in zip(obj.data, invalid)]
+                    obj_empty = _SbdfObject(_ValueTypeId.BOOL, invalid)
+                    va_empty = _ValueArray(_ValueArrayEncoding.BIT_ARRAY, obj_empty)
+                    cslice.add_property(_ColumnSlice.ValueProperty_IsInvalid, va_empty)
             tslice.add(cslice)
         tslice.write(file)
         row_offset += slice_row_count
@@ -713,36 +729,59 @@ class _SbdfObject:
         """writes the object to the specified file. valuetype information is not written"""
         self._write_n(file, 1, False)
 
+    # pylint: disable=too-many-branches
     def _write_n(self, file: typing.BinaryIO, n: int, packed: bool) -> None:
         valtype = _ValueType(self.valuetype)
         if valtype.is_array():
             byte_size = 0
+            # packed: no need to write 7bit packed int32
             if packed:
-                saved_bytes = []
-                for i in range(n):
-                    saved_bytes.append(valtype.to_bytes(self.data[i]))
-                    length = len(saved_bytes[i])
-                    byte_size += _get_7bit_packed_length(length) + length
-                _write_int32(file, byte_size)
-                for i in range(n):
-                    length = len(saved_bytes[i])
-                    _write_7bit_packed_int32(file, length)
-                    if length:
-                        _write_bytes(file, saved_bytes[i])
+                if isinstance(self.data, pd.Series) and self.valuetype is not _ValueTypeId.BINARY:
+                    barr = self.data.values.astype('U')
+                    _write_int32(file, sum(_get_7bit_packed_length(len(s.encode("utf-8"))) +
+                                            len(s.encode("utf-8")) for s in barr))
+                    for bstr in barr:
+                        bstr = bstr.encode("utf-8")
+                        _write_7bit_packed_int32(file, len(bstr))
+                        if len(bstr):
+                            _write_bytes(file, bstr)
+                else:
+                    saved_bytes = []
+                    for i in range(n):
+                        saved_bytes.append(valtype.to_bytes(self.data[i]))
+                        length = len(saved_bytes[i])
+                        byte_size += _get_7bit_packed_length(length) + length
+                    _write_int32(file, byte_size)
+                    for i in range(n):
+                        length = len(saved_bytes[i])
+                        _write_7bit_packed_int32(file, length)
+                        if length:
+                            _write_bytes(file, saved_bytes[i])
             else:
-                for i in range(n):
-                    valtype_bytes = valtype.to_bytes(self.data[i])
-                    length = len(valtype_bytes)
-                    _write_int32(file, length)
-                    if length:
-                        _write_bytes(file, valtype_bytes)
+                if isinstance(self.data, pd.Series):
+                    barr = self.data.values.astype('S')
+                    for bstr in barr:
+                        _write_7bit_packed_int32(file, len(bstr))
+                        if len(bstr):
+                            _write_bytes(file, bstr)
+                else:
+                    for i in range(n):
+                        valtype_bytes = valtype.to_bytes(self.data[i])
+                        length = len(valtype_bytes)
+                        _write_int32(file, length)
+                        if length:
+                            _write_bytes(file, valtype_bytes)
         else:
             size = valtype.get_packed_size()
             if size is None:
                 raise SBDFError("unknown typeid")
-            for i in range(n):
-                valtype_bytes = valtype.to_bytes(self.data[i])
-                _write_bytes(file, valtype_bytes)
+
+            if isinstance(self.data, pd.Series) and isinstance(self.data.values, np.ndarray):
+                _write_bytes(file, self.data.values.tobytes())
+            else:
+                for i in range(n):
+                    valtype_bytes = valtype.to_bytes(self.data[i])
+                    _write_bytes(file, valtype_bytes)
 
     @classmethod
     def read_array(cls, file: typing.BinaryIO, valuetype: '_ValueType') -> '_SbdfObject':
@@ -979,7 +1018,10 @@ class _ValueTypeId(enum.IntEnum):
     def infer_from_type(values, value_description: str) -> '_ValueTypeId':
         """determine the proper valuetype id from the Python types in a column"""
         # Remove any None (or other none-ish things) from values
-        vals = [x for x in values if not pd.isnull(x)]
+        if isinstance(values, pd.Series):
+            vals = values.dropna().tolist()
+        else:
+            vals = [x for x in values if not pd.isnull(x)]
         # Check if any values remain
         if not vals:
             raise SBDFError(f"cannot determine type for {value_description}; all values are missing")
@@ -1015,7 +1057,7 @@ class _ValueTypeId(enum.IntEnum):
         """determine the proper valuetype id from the Pandas dtype of a series"""
         dtype = series.dtype.name
         if dtype == "object":
-            return _ValueTypeId.infer_from_type(series.tolist(), series_description)
+            return _ValueTypeId.infer_from_type(series, series_description)
         if dtype == "category":
             return _ValueTypeId.infer_from_dtype(series.astype(series.cat.categories.dtype), series_description)
         typeid = {
@@ -1028,6 +1070,7 @@ class _ValueTypeId(enum.IntEnum):
             "float64": _ValueTypeId.DOUBLE,
             "datetime64[ns]": _ValueTypeId.DATETIME,
             "timedelta64[ns]": _ValueTypeId.TIMESPAN,
+            "string": _ValueTypeId.STRING,
         }.get(dtype, None)
         if typeid is None:
             raise SBDFError(f"unknown dtype '{dtype}' in {series_description}")
@@ -1228,10 +1271,37 @@ class _ValueType:
         return bitstring.pack('uintle:96,pad:17,bits:7,bool,bits:7', coefficient, biased_exponent_bits_high,
                               dec.sign, biased_exponent_bits_low).bytes
 
+    # pylint: disable=no-else-return,too-many-return-statements
     def to_bytes(self, obj: typing.Any) -> bytes:
         """return a SBDF representation of the Python objects"""
         try:
-            return getattr(self, "_to_bytes_" + self.type_id.name.lower(), lambda x: None)(obj)
+            name = self.type_id
+            if name == _ValueTypeId.BOOL:
+                return self._to_bytes_bool(obj)
+            elif name == _ValueTypeId.INT:
+                return self._to_bytes_int(obj)
+            elif name == _ValueTypeId.LONG:
+                return self._to_bytes_long(obj)
+            elif name == _ValueTypeId.FLOAT:
+                return self._to_bytes_float(obj)
+            elif name == _ValueTypeId.DOUBLE:
+                return self._to_bytes_double(obj)
+            elif name == _ValueTypeId.DATETIME:
+                return self._to_bytes_datetime(obj)
+            elif name == _ValueTypeId.DATE:
+                return self._to_bytes_date(obj)
+            elif name == _ValueTypeId.TIME:
+                return self._to_bytes_time(obj)
+            elif name == _ValueTypeId.TIMESPAN:
+                return self._to_bytes_timespan(obj)
+            elif name == _ValueTypeId.STRING:
+                return self._to_bytes_string(obj)
+            elif name == _ValueTypeId.BINARY:
+                return obj
+            elif name == _ValueTypeId.DECIMAL:
+                return self._to_bytes_decimal(obj)
+            else:
+                return None
         except (struct.error, bitstring.CreationError, UnicodeError) as exc:
             raise SBDFError(f"cannot convert '{obj}' to Spotfire {self.type_id.to_typename_string()} \
             type; value is outside representable range") from exc
@@ -1449,7 +1519,7 @@ if gpd is not None:
         # Convert to dataframe
         geom = [shapely.wkb.dumps(x) for x in gdf["geometry"]]
         geom_series = gdf["geometry"]
-        gdf = gdf.drop("geometry", 1)
+        gdf = gdf.drop(columns="geometry")
         names = gdf.keys()
         dframe = pd.DataFrame(gdf.to_numpy(), columns=names)
 
@@ -1478,10 +1548,13 @@ if gpd is not None:
         else:
             raise SBDFError("cannot convert collections of Shapely objects")
         if gdf.crs is not None:
-            if gdf.crs.startswith("+init="):
-                gdf.crs = gdf.crs[6:]
-            table_metadata["MapChart.GeographicCrs"] = gdf.crs
-
+            try:
+                table_metadata["MapChart.GeographicCrs"] = gdf.crs.to_string()
+            except AttributeError:
+                # GeoPandas <= 0.6.3 compatibility
+                if gdf.crs.startswith("+init="):
+                    gdf.crs = gdf.crs[6:]
+                table_metadata["MapChart.GeographicCrs"] = gdf.crs
         return dframe
 
 
@@ -1492,15 +1565,20 @@ if gpd is not None:
 
         # Convert to geodataframe
         geom = [shapely.wkb.loads(x) for x in dframe["Geometry"]]
-        dframe = dframe.drop("Geometry", 1)
+        dframe = dframe.drop(columns="Geometry")
         gdf = gpd.GeoDataFrame(dframe, geometry=geom)
 
         # Decide what CRS to use
         if "MapChart.GeographicCrs" in table_metadata.keys() and table_metadata["MapChart.GeographicCrs"] != "":
             proj = table_metadata["MapChart.GeographicCrs"][0]
-            if not proj.startswith("+init="):
-                proj = "+init=" + proj
             gdf.crs = proj
+            try:
+                # GeoPandas <= 0.6.3 compatibility
+                if not gdf.crs.startswith("+init="):
+                    gdf.crs = "+init=" + proj
+            except AttributeError:
+                pass
+
         return gdf
 
 if matplotlib is not None:
